@@ -14,86 +14,13 @@ import fnmatch
 import subprocess
 import functools
 import re
-import json
 
 import apt_pkg
 
 from UbuntuDrivers import kerneldetection
-from functools import cmp_to_key
 
 system_architecture = ''
 lookup_cache = {}
-custom_supported_gpus_json = '/etc/custom_supported_gpus.json'
-
-
-class NvidiaPkgNameInfo(object):
-    '''Class to process NVIDIA package names'''
-    def __init__(self, pkg_name):
-        self._pkg_name = pkg_name
-        self._obsolete_name_scheme = False
-        self._server = False
-        self._open = False
-        self._major_ver = -1
-        self._flavour = ''
-        self.is_valid = False
-        self._process_name(self._pkg_name)
-
-    def _process_name(self, name):
-        if 'nvidia' not in name:
-            logging.debug('NvidiaPkgNameInfo: %s is not an NVIDIA package. Skipping', name)
-            return
-
-        pattern = re.compile('nvidia-([0-9]+)')
-        match = pattern.match(name)
-
-        # Obsolete naming such as nvidia-340
-        if match:
-            self._obsolete_name_scheme = True
-            self._major_ver = int(match.group(1))
-            self._flavour = self._major_ver
-            self.is_valid = True
-            return
-
-        # Recent naming such as nvidia-driver-525
-        pattern = re.compile('nvidia-driver-([0-9]+)(.*)')
-        match = pattern.match(name)
-
-        if match:
-            self._server = match.group(0).find('-server') != -1
-            self._open = match.group(0).find('-open') != -1
-            self._flavour = '%s%s%s' % (match.group(1),
-                                        '-server' if self._server else '',
-                                        '-open' if self._open else '')
-            self._major_ver = int(match.group(1))
-            self.is_valid = True
-
-        # Recent naming such as nvidia-headless[-no-dkms]-525 for gpgpu
-        pattern = re.compile('nvidia-headless(.*)-([0-9]+)(.*)')
-        match = pattern.match(name)
-
-        if match:
-            self._server = match.group(0).find('-server') != -1
-            self._open = match.group(0).find('-open') != -1
-            self._flavour = '%s%s%s' % (match.group(2),
-                                        '-server' if self._server else '',
-                                        '-open' if self._open else '')
-            self._major_ver = match.group(2)
-            self.is_valid = True
-
-    def has_obsolete_name_scheme(self):
-        return self._obsolete_name_scheme
-
-    def is_server(self):
-        return self._server
-
-    def is_open(self):
-        return self._open
-
-    def get_major_version(self):
-        return self._major_ver
-
-    def get_flavour(self):
-        return self._flavour
 
 
 def get_apt_arch():
@@ -263,37 +190,6 @@ def _apt_cache_modalias_map(apt_cache):
     return result2
 
 
-def path_get_custom_supported_gpus():
-    return custom_supported_gpus_json
-
-
-def package_get_nv_allowing_driver(did):
-    '''Get nvidia allowing driver for specific devices.
-
-    did: 0x1234
-    Return the situable nvidia driver version for it.
-    '''
-    path = path_get_custom_supported_gpus()
-    version = None
-    try:
-        with open(path, "r") as stream:
-            try:
-                gpus = list(json.load(stream)['chips'])
-                for gpu in gpus:
-                    if gpu['devid'] == did:
-                        version = gpu['branch'].split('.')[0]
-                        logging.info("Found a specific nv driver version %s for %s(%s)" %
-                                     (version, gpu['name'], did))
-                        break
-            except Exception:
-                logging.debug('package_get_nv_allowing_driver(): unexpected json detected.')
-                pass
-    except Exception:
-        logging.debug('package_get_nv_allowing_driver(): unable to read %s' % path)
-        pass
-    return version
-
-
 def packages_for_modalias(apt_cache, modalias):
     '''Search packages which match the given modalias.
 
@@ -309,24 +205,8 @@ def packages_for_modalias(apt_cache, modalias):
         packages_for_modalias.cache_maps[apt_cache_hash] = cache_map
 
     pat, bus_map = cache_map.get(modalias.split(':', 1)[0], (None, {}))
-    vid, did = _get_vendor_model_from_alias(modalias)
-    nvamd = None
-    found = 0
-    if vid == "10DE":
-        nvamd = package_get_nv_allowing_driver("0x" + did)
-        nvamdn = "nvidia-driver-%s" % nvamd
-        nvamda = "pci:v000010DEd0000%s*" % did
-        for p in apt_cache.packages:
-            if p.get_fullname().split(':')[0] == nvamdn:
-                bus_map[nvamda] = set([nvamdn])
-                found = 1
-        if nvamd is not None and not found:
-            logging.debug('%s is not in the package pool.' % nvamdn)
-
-    if not found:
-        if pat is None or not pat.match(modalias):
-            return []
-
+    if pat is None or not pat.match(modalias):
+        return []
     for alias in bus_map:
         if fnmatch.fnmatchcase(modalias.lower(), alias.lower()):
             for p in bus_map[alias]:
@@ -379,27 +259,6 @@ def _is_package_from_distro(apt_cache, pkg):
         return False
 
 
-def _pkg_get_open_preference(apt_cache, pkg):
-    '''Determine if -open package is prefered from apt Package object'''
-    depcache = apt_pkg.DepCache(apt_cache)
-    candidate = depcache.get_candidate_ver(pkg)
-    records = apt_pkg.PackageRecords(apt_cache)
-    records.lookup(candidate.file_list[0])
-
-    try:
-        preference = records['Prefer-Variant']
-    except (KeyError, AttributeError):
-        logging.debug('_pkg_get_open_preference %s: package has Prefer-Open header, cannot determine', pkg.name)
-        return False
-
-    if preference not in ('Open', 'Closed'):
-        logging.debug('_pkg_get_open_preference %s: package has invalid Prefer-Open %s'
-                      'header, cannot determine preference', pkg.name, preference)
-        return None
-
-    return preference
-
-
 def _pkg_get_module(apt_cache, pkg):
     '''Determine module name from apt Package object'''
     depcache = apt_pkg.DepCache(apt_cache)
@@ -444,46 +303,10 @@ def _pkg_get_support(apt_cache, pkg):
     return support
 
 
-def _is_nv_allowing_runtimepm_supported(alias, ver):
-    '''alias: e.g. pci:v000010DEd000024BAsv0000103Csd000089C6bc03sc00i00'''
-    result = re.search('pci:v0000(.*)d0000(.*)sv(.*)', alias)
-    vid = result.group(1)
-    did = result.group(2)
-    if vid != "10DE":
-        return False
-    did = "0x%s" % did
-    path = path_get_custom_supported_gpus()
-    try:
-        with open(path, "r") as stream:
-            try:
-                gpus = list(json.load(stream)['chips'])
-                for gpu in gpus:
-                    if gpu['devid'] == did and 'runtimepm' in gpu['features']:
-                        if gpu['branch'].split('.')[0] != ver:
-                            logging.debug('Candidate version does not match %s != %s' %
-                                          (gpu['branch'].split('.')[0], ver))
-                            return False
-                        logging.info("Found runtimepm supports on %s." % did)
-                        return True
-            except Exception:
-                logging.debug('_is_nv_allowing_runtimepm_supported(): unexpected json detected')
-                pass
-    except Exception:
-        logging.debug('_is_nv_allowing_runtimepm_supported(): unable to read %s' % path)
-        pass
-    return False
-
-
-def _is_runtimepm_supported(apt_cache, pkg, alias):
+def _is_runtimepm_supported(pkg, alias):
     '''Check if the package supports runtimepm for the given modalias'''
     try:
-        depcache = apt_pkg.DepCache(apt_cache)
-        candidate = depcache.get_candidate_ver(pkg)
-        records = apt_pkg.PackageRecords(apt_cache)
-        records.lookup(candidate.file_list[0])
-        section = apt_pkg.TagSection(records.record)
-        ver = candidate.ver_str.split('.')[0]
-        m = section['PmAliases']
+        m = pkg.candidate.record['PmAliases']
     except (KeyError, AttributeError, UnicodeDecodeError):
         return False
     else:
@@ -492,8 +315,6 @@ def _is_runtimepm_supported(apt_cache, pkg, alias):
 
         n = m[m.find('(')+1: m.find(')')]
         modaliases = n.split(', ')
-        if _is_nv_allowing_runtimepm_supported(alias, ver):
-            return True
         return any(fnmatch.fnmatch(alias.lower(), regex.lower()) for regex in modaliases)
 
 
@@ -565,24 +386,6 @@ def _get_db_name(syspath, alias):
     return (vendor, model)
 
 
-def _is_open_prefered(apt_cache, pkg):
-    if not pkg.name.startswith('nvidia-'):
-        return False
-
-    preference = _pkg_get_open_preference(apt_cache, pkg)
-    if preference is None:
-        nvidia_info = NvidiaPkgNameInfo(pkg.name)
-        if nvidia_info.get_major_version() >= 560:
-            logging.debug('_is_open_prefered(%s): True', pkg.name)
-            return True
-    elif preference == 'Open':
-        logging.debug('_is_open_prefered(%s): True', pkg.name)
-        return True
-
-    logging.debug('_is_open_prefered(%s): False', pkg.name)
-    return False
-
-
 def set_nvidia_kms(value):
     '''Set KMS on or off for NVIDIA'''
     nvidia_kms_file = '/lib/modprobe.d/nvidia-kms.conf'
@@ -653,8 +456,7 @@ def system_driver_packages(apt_cache=None, sys_path=None, freeonly=False, includ
                     'free': _is_package_free(apt_cache, p),
                     'from_distro': _is_package_from_distro(apt_cache, p),
                     'support': _pkg_get_support(apt_cache, p),
-                    'runtimepm': _is_runtimepm_supported(apt_cache, p, alias),
-                    'open_preferred': _is_open_prefered(apt_cache, p)
+                    'runtimepm': _is_runtimepm_supported(p, alias)
                 }
             (vendor, model) = _get_db_name(syspath, alias)
             if vendor is not None:
@@ -713,16 +515,19 @@ def _get_headless_no_dkms_metapackage(pkg, apt_cache):
     depcache = apt_pkg.DepCache(apt_cache)
     name = pkg.name
 
-    nvidia_info = NvidiaPkgNameInfo(name)
-    if not nvidia_info.is_valid:
-        logging.debug('Unsupported driver detected: %s. Skipping' % name)
+    pattern = re.compile('nvidia-([0-9]+)')
+    match = pattern.match(name)
+    if match:
+        logging.debug('Legacy driver detected: %s. Skipping.', name)
         return metapackage
 
-    if nvidia_info.has_obsolete_name_scheme():
-        logging.debug('Legacy driver detected: %s. Skipping.' % name)
+    pattern = re.compile('nvidia-driver-([0-9]+)(.*)')
+    match = pattern.match(name)
+    if not match:
+        logging.debug('No flavour can be found in %s. Skipping.', name)
         return metapackage
 
-    candidate_flavour = nvidia_info.get_flavour()
+    candidate_flavour = '%s%s' % (match.group(1), match.group(2))
     candidate = 'nvidia-headless-no-dkms-%s' % (candidate_flavour)
 
     try:
@@ -795,7 +600,6 @@ def system_device_specific_metapackages(apt_cache=None, sys_path=None, include_o
                     'from_distro': _is_package_from_distro(apt_cache, p),
                     'recommended': True,
                     'support': _pkg_get_support(apt_cache, p),
-                    'open_preferred': _is_open_prefered(apt_cache, p),
                 }
 
     return packages
@@ -857,7 +661,6 @@ def system_gpgpu_driver_packages(apt_cache=None, sys_path=None):
                         'free': _is_package_free(apt_cache, p),
                         'from_distro': _is_package_from_distro(apt_cache, p),
                         'support': _pkg_get_support(apt_cache, p),
-                        'open_preferred': _is_open_prefered(apt_cache, p),
                     }
                 if vendor is not None:
                     packages[p.name]['vendor'] = vendor
@@ -976,14 +779,12 @@ def system_device_drivers(apt_cache=None, sys_path=None, freeonly=False):
     return result
 
 
-def get_desktop_package_list(
-        apt_cache, sys_path=None, free_only=False, include_oem=True,
-        driver_string='', include_dkms=False):
+def get_desktop_package_list(apt_cache, sys_path=None, free_only=False, include_oem=True, driver_string=''):
     '''Return the list of packages that should be installed'''
     packages = system_driver_packages(
         apt_cache, sys_path, freeonly=free_only,
         include_oem=include_oem)
-    packages = auto_install_filter(packages, driver_string, get_recommended=False)
+    packages = auto_install_filter(packages, driver_string)
     if not packages:
         logging.debug('No drivers found for installation.')
         return packages
@@ -993,15 +794,13 @@ def get_desktop_package_list(
 
     # ignore packages which are already installed
     to_install = []
-    for p, _ in sorted(packages.items(),
-                       key=cmp_to_key(lambda left, right: _cmp_gfx_alternatives(left[0], right[0])),
-                       reverse=True):
+    for p in packages:
         package_obj = apt_cache[p]
         if not package_obj.current_ver:
+            to_install.append(p)
 
             candidate = depcache.get_candidate_ver(package_obj)
             records.lookup(candidate.file_list[0])
-            to_install.append(p)
 
             # See if runtimepm is supported
             if records['runtimepm']:
@@ -1018,11 +817,7 @@ def get_desktop_package_list(
             try:
                 modules_package = get_linux_modules_metapackage(apt_cache, p)
                 if modules_package and not apt_cache[modules_package].current_ver:
-                    if not include_dkms and "dkms" in modules_package:
-                        to_install.remove(p)
-                        continue
                     to_install.append(modules_package)
-                    break
             except KeyError:
                 pass
 
@@ -1031,12 +826,22 @@ def get_desktop_package_list(
 
 def nvidia_desktop_pre_installation_hook(to_install):
     '''Applies changes that need to happen before installing the NVIDIA drivers'''
+    with_nvidia_kms = False
+
     # Enable KMS if nvidia >= 470
     for package_name in to_install:
-        nvidia_info = NvidiaPkgNameInfo(package_name)
-        if nvidia_info.get_major_version() >= 470:
-            set_nvidia_kms(1)
-            return
+        pattern = re.compile('nvidia-driver-([0-9]+)(.*)')
+        match = pattern.match(package_name)
+        if match:
+            try:
+                version = int(match.group(1))
+            except ValueError:
+                pass
+            finally:
+                with_nvidia_kms = version >= 470
+
+    if with_nvidia_kms:
+        set_nvidia_kms(1)
 
 
 def nvidia_desktop_post_installation_hook():
@@ -1092,7 +897,7 @@ def _process_driver_string(string):
     return driver
 
 
-def gpgpu_install_filter(packages, drivers_str, get_recommended=True):
+def gpgpu_install_filter(packages, drivers_str):
     drivers = []
     allow = []
     result = {}
@@ -1185,17 +990,14 @@ def gpgpu_install_filter(packages, drivers_str, get_recommended=True):
                     result[p] = packages[p]
                 else:
                     # print('before recommended: %s' % packages[p])
-                    if get_recommended:
-                        if packages[p].get('recommended'):
-                            result[p] = packages[p]
-                    else:
+                    if packages[p].get('recommended'):
                         result[p] = packages[p]
                         # print('Found "recommended" flavour in %s' % (packages[p]))
                 break
     return result
 
 
-def auto_install_filter(packages, drivers_str='', get_recommended=True):
+def auto_install_filter(packages, drivers_str=''):
     '''Get packages which are appropriate for automatic installation.
 
     Return the subset of the given list of packages which are appropriate for
@@ -1219,10 +1021,7 @@ def auto_install_filter(packages, drivers_str='', get_recommended=True):
 
     result = {}
     for p in allow:
-        if get_recommended:
-            if 'recommended' not in packages[p] or packages[p]['recommended']:
-                result[p] = packages[p]
-        else:
+        if 'recommended' not in packages[p] or packages[p]['recommended']:
             result[p] = packages[p]
     return result
 
@@ -1296,140 +1095,66 @@ def _pkg_support_from_cache(x):
     return None
 
 
-def _pkg_open_preferred_from_cache(x):
-    '''Look up driver and return if open package is preferred'''
-    if lookup_cache.get(x):
-        return lookup_cache.get(x).get('open_preferred')
-    return False
-
-
-def _get_fit_level(x):
-    '''decide how well the package fits desktop environment'''
-
-    # for desktop non-server packages are prefered to -server ones
-    # if a package preferes open variants, these come on top
-
-    if _pkg_open_preferred_from_cache(x):
-        if re.search(r'(\d{3})-open$', x):
-            return 3
-
-        if re.search(r'(\d{3})$', x):
-            return 2
-
-        if re.search(r'(\d{3})-server-open$', x):
-            return 1
-
-        if re.search(r'(\d{3})-server$', x):
-            return 0
-    else:
-        if re.search(r'(\d{3})$', x):
-            return 3
-
-        if re.search(r'(\d{3})-server$', x):
-            return 2
-
-        if re.search(r'(\d{3})-open$', x):
-            return 1
-
-        if re.search(r'(\d{3})-server-open$', x):
-            return 0
-    return 0
-
-
-def _get_fit_level_gpgpu(x):
-    '''decide how well the package fits server environment'''
-
-    # for server environment -server packages are prefered to non-server ones
-    # if a package preferes open variants, these come on top
-
-    if _pkg_open_preferred_from_cache(x):
-        if re.search(r'(\d{3})-server-open$', x):
-            return 3
-
-        if re.search(r'(\d{3})-server$', x):
-            return 2
-
-        if re.search(r'(\d{3})-open$', x):
-            return 1
-
-        if re.search(r'(\d{3})$', x):
-            return 0
-    else:
-        if re.search(r'(\d{3})-server$', x):
-            return 3
-
-        if re.search(r'(\d{3})$', x):
-            return 2
-
-        if re.search(r'(\d{3})-server-open$', x):
-            return 1
-
-        if re.search(r'(\d{3})-open$', x):
-            return 0
-    return 0
-
-
 def _cmp_gfx_alternatives(x, y):
     '''Compare two graphics driver names in terms of preference. (desktop)
 
-    -open by default sorts after non-open, unless a driver preferes open variant
+    -open always sorts after non-open.
     -server always sorts after non-server.
     LTSB (Long Term Support Branch) always sorts before NFB (New Feature Branch).
     Legacy always sorts before Beta.
     '''
 
-    if x.startswith('nvidia-') and y.startswith('nvidia-'):
-        if _get_fit_level(x) > _get_fit_level(y):
+    if x.endswith('-open') and not y.endswith('-open'):
+        return -1
+    if not x.endswith('-open') and y.endswith('-open'):
+        return 1
+
+    if x.endswith('-server') and not y.endswith('-server'):
+        return -1
+    if not x.endswith('-server') and y.endswith('-server'):
+        return 1
+
+    preferred_support = ['PB', 'LTSB']
+
+    x_score = 0
+    y_score = 0
+
+    x_support = _pkg_support_from_cache(x)
+    y_support = _pkg_support_from_cache(y)
+
+    if x_support in preferred_support:
+        x_score += 100
+
+    if y_support in preferred_support:
+        y_score += 100
+
+    if x > y:
+        x_score += 1
+    elif x < y:
+        y_score += 1
+
+    if ((x_score >= 100) or (y_score >= 100)):
+        if x_score > y_score:
             return 1
-        if _get_fit_level(x) < _get_fit_level(y):
+        elif x_score < y_score:
             return -1
 
-        preferred_support = ['PB', 'LTSB']
-
-        x_score = 0
-        y_score = 0
-
-        x_support = _pkg_support_from_cache(x)
-        y_support = _pkg_support_from_cache(y)
-
-        if x_support in preferred_support:
-            x_score += 100
-
-        if y_support in preferred_support:
-            y_score += 100
-
-        if x > y:
-            x_score += 1
-        elif x < y:
-            y_score += 1
-
-        if ((x_score >= 100) or (y_score >= 100)):
-            if x_score > y_score:
-                return 1
-            elif x_score < y_score:
-                return -1
-
-        if _pkg_support_from_cache(x) == 'PB' and _pkg_support_from_cache(y) != 'PB':
-            return 1
-        if _pkg_support_from_cache(x) != 'PB' and _pkg_support_from_cache(y) == 'PB':
-            return -1
-        if _pkg_support_from_cache(x) == 'LTSB' and _pkg_support_from_cache(y) != 'LTSB':
-            return 1
-        if _pkg_support_from_cache(x) != 'LTSB' and _pkg_support_from_cache(y) == 'LTSB':
-            return -1
-        if _pkg_support_from_cache(x) == 'NFB' and _pkg_support_from_cache(y) != 'NFB':
-            return -1
-        if _pkg_support_from_cache(x) != 'NFB' and _pkg_support_from_cache(y) == 'NFB':
-            return 1
-        if _pkg_support_from_cache(x) == 'Legacy' and _pkg_support_from_cache(y) != 'Legacy':
-            return -1
-        if _pkg_support_from_cache(x) != 'Legacy' and _pkg_support_from_cache(y) == 'Legacy':
-            return 1
-        if _pkg_support_from_cache(x) == 'Beta' and _pkg_support_from_cache(y) != 'Beta':
-            return -1
-        if _pkg_support_from_cache(x) != 'Beta' and _pkg_support_from_cache(y) == 'Beta':
-            return 1
-
+    if _pkg_support_from_cache(x) == 'PB' and _pkg_support_from_cache(y) != 'PB':
+        return 1
+    if _pkg_support_from_cache(x) != 'PB' and _pkg_support_from_cache(y) == 'PB':
+        return -1
+    if _pkg_support_from_cache(x) == 'LTSB' and _pkg_support_from_cache(y) != 'LTSB':
+        return 1
+    if _pkg_support_from_cache(x) != 'LTSB' and _pkg_support_from_cache(y) == 'LTSB':
+        return -1
+    if _pkg_support_from_cache(x) == 'Legacy' and _pkg_support_from_cache(y) != 'Legacy':
+        return -1
+    if _pkg_support_from_cache(x) != 'Legacy' and _pkg_support_from_cache(y) == 'Legacy':
+        return 1
+    if _pkg_support_from_cache(x) == 'Beta' and _pkg_support_from_cache(y) != 'Beta':
+        return -1
+    if _pkg_support_from_cache(x) != 'Beta' and _pkg_support_from_cache(y) == 'Beta':
+        return 1
     if x < y:
         return -1
     if x > y:
@@ -1441,40 +1166,31 @@ def _cmp_gfx_alternatives(x, y):
 def _cmp_gfx_alternatives_gpgpu(x, y):
     '''Compare two graphics driver names in terms of preference. (server)
 
-    -open by default sorts after non-open, unless a driver preferes open variant
     -server always sorts before non-server.
-    non server package always sorts after server package
     LTSB (Long Term Support Branch) always sorts before NFB (New Feature Branch).
     Legacy always sorts before Beta.
     '''
+    if x.endswith('-server') and not y.endswith('-server'):
+        return 1
+    if not x.endswith('-server') and y.endswith('-server'):
+        return -1
 
-    if x.startswith('nvidia-') and y.startswith('nvidia-'):
-        if _get_fit_level_gpgpu(x) > _get_fit_level_gpgpu(y):
-            return 1
-        if _get_fit_level_gpgpu(x) < _get_fit_level_gpgpu(y):
-            return -1
-
-        if _pkg_support_from_cache(x) == 'PB' and _pkg_support_from_cache(y) != 'PB':
-            return 1
-        if _pkg_support_from_cache(x) != 'PB' and _pkg_support_from_cache(y) == 'PB':
-            return -1
-        if _pkg_support_from_cache(x) == 'LTSB' and _pkg_support_from_cache(y) != 'LTSB':
-            return 1
-        if _pkg_support_from_cache(x) != 'LTSB' and _pkg_support_from_cache(y) == 'LTSB':
-            return -1
-        if _pkg_support_from_cache(x) == 'Legacy' and _pkg_support_from_cache(y) != 'Legacy':
-            return -1
-        if _pkg_support_from_cache(x) != 'Legacy' and _pkg_support_from_cache(y) == 'Legacy':
-            return 1
-        if _pkg_support_from_cache(x) == 'NFB' and _pkg_support_from_cache(y) != 'NFB':
-            return -1
-        if _pkg_support_from_cache(x) != 'NFB' and _pkg_support_from_cache(y) == 'NFB':
-            return 1
-        if _pkg_support_from_cache(x) == 'Beta' and _pkg_support_from_cache(y) != 'Beta':
-            return -1
-        if _pkg_support_from_cache(x) != 'Beta' and _pkg_support_from_cache(y) == 'Beta':
-            return 1
-
+    if _pkg_support_from_cache(x) == 'PB' and _pkg_support_from_cache(y) != 'PB':
+        return 1
+    if _pkg_support_from_cache(x) != 'PB' and _pkg_support_from_cache(y) == 'PB':
+        return -1
+    if _pkg_support_from_cache(x) == 'LTSB' and _pkg_support_from_cache(y) != 'LTSB':
+        return 1
+    if _pkg_support_from_cache(x) != 'LTSB' and _pkg_support_from_cache(y) == 'LTSB':
+        return -1
+    if _pkg_support_from_cache(x) == 'Legacy' and _pkg_support_from_cache(y) != 'Legacy':
+        return -1
+    if _pkg_support_from_cache(x) != 'Legacy' and _pkg_support_from_cache(y) == 'Legacy':
+        return 1
+    if _pkg_support_from_cache(x) == 'Beta' and _pkg_support_from_cache(y) != 'Beta':
+        return -1
+    if _pkg_support_from_cache(x) != 'Beta' and _pkg_support_from_cache(y) == 'Beta':
+        return 1
     if x < y:
         return -1
     if x > y:
@@ -1570,13 +1286,14 @@ def get_linux_modules_metapackage(apt_cache, candidate):
 
     depcache = apt_pkg.DepCache(apt_cache)
 
-    nvidia_info = NvidiaPkgNameInfo(candidate)
-    if not nvidia_info.is_valid:
-        logging.debug('Non NVIDIA linux-modules packages are not supported at this time: %s. Skipping' % candidate)
+    if 'nvidia' not in candidate:
+        logging.debug('Non NVIDIA linux-modules packages are not supported at this time: %s. Skipping', candidate)
         return metapackage
 
-    if nvidia_info.has_obsolete_name_scheme():
-        logging.debug('Legacy driver detected: %s. Skipping.' % candidate)
+    pattern = re.compile('nvidia-([0-9]+)')
+    match = pattern.match(candidate)
+    if match:
+        logging.debug('Legacy driver detected: %s. Skipping.', candidate)
         return metapackage
 
     linux_image_meta = get_linux_image(apt_cache)
@@ -1586,10 +1303,16 @@ def get_linux_modules_metapackage(apt_cache, candidate):
     if linux_image:
         linux_flavour = linux_image.replace('linux-image-', '')
     else:
-        logging.debug('No linux-image can be found for %s. Skipping.' % candidate)
+        logging.debug('No linux-image can be found for %s. Skipping.', candidate)
         return metapackage
 
-    candidate_flavour = nvidia_info.get_flavour()
+    pattern = re.compile('nvidia-.*-([0-9]+)(.*)')
+    match = pattern.match(candidate)
+    if not match:
+        logging.debug('No flavour can be found in %s. Skipping.', candidate)
+        return metapackage
+
+    candidate_flavour = '%s%s' % (match.group(1), match.group(2))
     linux_modules_candidate = 'linux-modules-nvidia-%s-%s' % (candidate_flavour, linux_flavour)
 
     try:
@@ -1624,11 +1347,8 @@ def get_linux_modules_metapackage(apt_cache, candidate):
                         if dep.parent_pkg.name.startswith('linux-modules-nvidia-')]
 
         pick = ''
-        candidate_suffix = get_linux_image(apt_cache).replace('linux-image-', '')
-        if candidate_suffix == "virtual":
-            candidate_suffix = "generic"
-        modules_candidate = 'linux-modules-nvidia-%s-%s' % (candidate_flavour, candidate_suffix)
-
+        modules_candidate = 'linux-modules-nvidia-%s-%s' % (candidate_flavour,
+                                                            get_linux_image(apt_cache).replace('linux-image-', ''))
         for dep in reverse_deps:
             if dep == modules_candidate:
                 pick = dep
